@@ -4,10 +4,9 @@
 
   Authors: Sebastian Deorowicz, Agnieszka Debudaj-Grabysz, Marek Kokot
 
-  Version: 3.2.2
-  Date   : 2023-03-10
+  Version: 3.2.3
+  Date   : 2023-12-08
 */
-
 #include <algorithm>
 #include "defs.h"
 #include "fastq_reader.h"
@@ -979,9 +978,15 @@ void CFastqReaderDataSrc::init_stream()
 }
 
 //----------------------------------------------------------------------------------
-bool CFastqReaderDataSrc::pop_pack(uchar*& data, uint64& size, FilePart& file_part, CompressionType& mode)
+bool CFastqReaderDataSrc::pop_pack(uchar*& data, uint64& size, FilePart& file_part, CompressionType& mode, bool& last_in_file)
 {
-	end_reached = !binary_pack_queue->pop(in_data, in_data_size, file_part, compression_type);
+	end_reached = !binary_pack_queue->pop(data, size, file_part, mode);
+	if (file_part == FilePart::End)
+	{
+		last_in_file = true;
+		in_progress = false;
+		return false;
+	}
 	return !end_reached;
 }
 
@@ -1005,7 +1010,7 @@ uint64 CFastqReaderDataSrc::read(uchar* buff, uint64 size, bool& last_in_file, b
 	first_in_file = false;
 	if (!in_progress)
 	{
-		if (!pop_pack(in_data, in_data_size, file_part, compression_type))
+		if (!pop_pack(in_data, in_data_size, file_part, compression_type, last_in_file))
 			return 0;
 		in_progress = true;
 		first_in_file = true;
@@ -1017,15 +1022,30 @@ uint64 CFastqReaderDataSrc::read(uchar* buff, uint64 size, bool& last_in_file, b
 	{
 		stream.next_out = buff;
 		stream.avail_out = (uint32)size;
-		int ret;
+		int ret = Z_OK;
 		do
 		{
 			if (!stream.avail_in)
 			{
 				pmm_binary_file_reader->free(in_data);
 				in_data = nullptr;
-				if (!pop_pack(in_data, in_data_size, file_part, compression_type))
-					return 0;
+				if (!pop_pack(in_data, in_data_size, file_part, compression_type, last_in_file)) {
+					auto ret_val = size - stream.avail_out;
+
+					if (inflateEnd(&stream) != Z_OK) {
+						std::ostringstream ostr;
+						ostr << "Some error while reading gzip file (inflateEnd) in (" << __FILE__ << ": " << __LINE__ << ")";
+						CCriticalErrorHandler::Inst().HandleCriticalError(ostr.str());
+					}
+
+					if (ret != Z_STREAM_END) {
+						std::ostringstream ostr;
+						ostr << "Unexpected end of gzip file";
+						CCriticalErrorHandler::Inst().HandleCriticalError(ostr.str());
+					}
+
+					return ret_val;
+				}
 				stream.avail_in = (uint32)in_data_size;
 				stream.next_in = in_data;
 			}
@@ -1039,7 +1059,7 @@ uint64 CFastqReaderDataSrc::read(uchar* buff, uint64 size, bool& last_in_file, b
 			case Z_DATA_ERROR:
 			{
 				std::ostringstream ostr;
-				ostr << "Some error while reading gzip file";
+				ostr << "Some error while reading gzip file in (" << __FILE__ << ": " << __LINE__ << ")";
 				CCriticalErrorHandler::Inst().HandleCriticalError(ostr.str());
 			}
 			case Z_MEM_ERROR:
@@ -1053,14 +1073,19 @@ uint64 CFastqReaderDataSrc::read(uchar* buff, uint64 size, bool& last_in_file, b
 			{
 				uchar* tmp_data = nullptr;
 				uint64 tmp_size = 0;
-				bool multistream = stream.avail_in || binary_pack_queue->peek_next_pack(tmp_data, tmp_size);
+				bool multistream = stream.avail_in;
+
+				if (stream.avail_in < 2) {
+					bool peek_res = binary_pack_queue->peek_next_pack(tmp_data, tmp_size);
+					multistream = stream.avail_in || peek_res;
+				}
 				bool garbage = false;
 				if (multistream)
 				{
 					if (stream.avail_in + tmp_size < 2)
 					{
 						std::ostringstream ostr;
-						ostr << "Some error while reading gzip file";
+						ostr << "Some error while reading gzip file in (" << __FILE__ << ": " << __LINE__ << ")";
 						CCriticalErrorHandler::Inst().HandleCriticalError(ostr.str());
 					}
 					uchar b1, b2;
@@ -1087,10 +1112,14 @@ uint64 CFastqReaderDataSrc::read(uchar* buff, uint64 size, bool& last_in_file, b
 				{
 					pmm_binary_file_reader->free(in_data);
 					in_data = nullptr;
-					inflateEnd(&stream);
+					if (inflateEnd(&stream) != Z_OK) {
+						std::ostringstream ostr;
+						ostr << "Some error while reading gzip file (inflateEnd) in (" << __FILE__ << ": " << __LINE__ << ")";
+						CCriticalErrorHandler::Inst().HandleCriticalError(ostr.str());
+					}
 					in_progress = false;
 					//pull end
-					bool queue_end = !pop_pack(in_data, in_data_size, file_part, compression_type);
+					bool queue_end = !pop_pack(in_data, in_data_size, file_part, compression_type, last_in_file);
 					if (!queue_end && file_part != FilePart::End && !garbage)
 					{
 						std::ostringstream ostr;
@@ -1106,7 +1135,7 @@ uint64 CFastqReaderDataSrc::read(uchar* buff, uint64 size, bool& last_in_file, b
 							uchar* tmp;
 							uint64 tmpsize;
 							CompressionType tmpcomptype;
-							pop_pack(tmp, tmpsize, tmpfilepart, tmpcomptype);
+							pop_pack(tmp, tmpsize, tmpfilepart, tmpcomptype, last_in_file);
 						}
 					}
 					last_in_file = true;
@@ -1134,8 +1163,9 @@ uint64 CFastqReaderDataSrc::read(uchar* buff, uint64 size, bool& last_in_file, b
 			{
 				pmm_binary_file_reader->free(in_data);
 				in_data = nullptr;
-				pop_pack(in_data, in_data_size, file_part, compression_type);
-				if (file_part == FilePart::End)
+				//may be false even if file_part != FilePart::End in stats mode
+				auto pop_res = pop_pack(in_data, in_data_size, file_part, compression_type, last_in_file);
+				if (!pop_res || file_part == FilePart::End)
 				{
 					in_progress = false;
 					last_in_file = true;
