@@ -4,8 +4,8 @@
   
   Authors: Sebastian Deorowicz, Agnieszka Debudaj-Grabysz, Marek Kokot
   
-  Version: 3.2.2
-  Date   : 2023-03-10
+  Version: 3.2.3
+  Date   : 2023-12-08
 */
 
 #ifndef _KB_SORTER_H
@@ -1228,7 +1228,7 @@ template <unsigned SIZE> void CKmerBinSorter<SIZE>::CompactKmers()
 		{
 			n_cutoff_min++;
 		}
-		else if (count >= cutoff_max)
+		else if (count > cutoff_max)
 		{
 			n_cutoff_max++;
 		}
